#include "stdafx.h"
#pragma hdrstop

#include "CommonCollisionListener.h"
#include "entity/CollideClasses.h"
#include "entity/ThingEntity.h"
#include "entity/StaticEntity.h"
#include <unco/kernel/Contexts.h>
#include <dingus/physics-ode/PhysicsContext.h>


#include "entity/ContactEntity.h"
#include "level/Level.h"
#include "level/GameInfo.h"


void CCommonCollisionListener::onCollide( dingus::CCollidable& me, dingus::CCollidable& him )
{
	assert( me.getID() != him.getID() );

	unsigned long meCat = me.getCategoryBits();
	if( meCat & (1<<COL_ENTITY_STATIC) ) {
		collideStatic( me, him );
	} else if( meCat & (1<<COL_ENTITY_THING) ) {
		collideThing( me, him );
	} else if( meCat & (1<<COL_ENTITY_CAR) ) {
		collideCar( me, him );
	}
}


void CCommonCollisionListener::collideStatic( dingus::CCollidable& me, dingus::CCollidable& him )
{
	assert( me.getCategoryBits() & (1<<COL_ENTITY_STATIC) );

	unsigned long himCat = him.getCategoryBits();
	if( himCat & (1<<COL_ENTITY_THING) ) {
		collideStatic2Thing( me, him );
	} else if( himCat & (1<<COL_ENTITY_CAR) ) {
		collideStatic2Car( me, him );
	}
}

void CCommonCollisionListener::collideThing( dingus::CCollidable& me, dingus::CCollidable& him )
{
	assert( me.getCategoryBits() & (1<<COL_ENTITY_THING) );

	unsigned long himCat = him.getCategoryBits();
	if( himCat & (1<<COL_ENTITY_THING) ) {
		collideThing2Thing( me, him );
	} else if( himCat & (1<<COL_ENTITY_STATIC) ) {
		collideStatic2Thing( him, me );
	} else if( himCat & (1<<COL_ENTITY_CAR) ) {
		collideThing2Car( me, him );
	}
}

void CCommonCollisionListener::collideCar( dingus::CCollidable& me, dingus::CCollidable& him )
{
	assert( me.getCategoryBits() & (1<<COL_ENTITY_CAR) );

	unsigned long himCat = him.getCategoryBits();
	if( himCat & (1<<COL_ENTITY_THING) ) {
		collideThing2Car( him, me );
	} else if( himCat & (1<<COL_ENTITY_STATIC) ) {
		collideStatic2Car( him, me );
	} else if( himCat & (1<<COL_ENTITY_CAR) ) {
		collideCar2Car( me, him );
	}
}

int CCommonCollisionListener::commonCollide(
		dingus::CCollidable& me, dingus::CCollidable& him,
		int maxContacts, dContactGeom* contacts, dContact& c, bool isPhysMe, bool isPhysHim )
{
	int ncontacts = me.collideWith( him, maxContacts, contacts );
	if( !ncontacts )
		return ncontacts;

	dingus::TPhysicsableID physMe, physHim;
	if( isPhysMe ) {
		CBasePhysicsEntity* e = reinterpret_cast<CBasePhysicsEntity*>( me.getUserData() );
		assert( e );
		physMe = e->getPhysComp().getPhysicsable();
	} else
		physMe = NULL;
	if( isPhysHim ) {
		CBasePhysicsEntity* e = reinterpret_cast<CBasePhysicsEntity*>( him.getUserData() );
		assert( e );
		physHim = e->getPhysComp().getPhysicsable();
	} else
		physHim = NULL;

	dingus::CPhysicsContext& phys = unco::CContexts::getPhysicsCtx();
	for( int i = 0; i < ncontacts; ++i ) {
		c.geom = contacts[i];
		phys.addContactJoint( c, physMe, physHim );
		/*
		CContactEntity* ce = new CContactEntity( c.geom, CGameInfo::getInstance().getLevel().getVisStuff() );
		CGameInfo::getInstance().getLevel().getStuff().addEntity( ce );
		ce->attach();
		float scale = 0.5f + c.geom.depth * 15.0f;
		ce->getMatrix().getAxisX() *= scale;
		ce->getMatrix().getAxisY() *= scale;
		ce->getMatrix().getAxisZ() *= scale;
		*/
	}
	return ncontacts;
}


void CCommonCollisionListener::collideThing2Thing( dingus::CCollidable& me, dingus::CCollidable& him )
{
	assert( me.getCategoryBits() & (1<<COL_ENTITY_THING) );
	assert( him.getCategoryBits() & (1<<COL_ENTITY_THING) );

	const int MAX_CONTACTS = 3;
	dContactGeom contacts[MAX_CONTACTS];
	dContact c;
	c.surface.mode = dContactSoftCFM | dContactSoftERP;
	c.surface.mu = 250;
	c.surface.soft_erp = 0.2;
	c.surface.soft_cfm = 0.0003;
	int ncontacts = commonCollide( me, him, MAX_CONTACTS, contacts, c, true, true );
}

void CCommonCollisionListener::collideThing2Car( dingus::CCollidable& me, dingus::CCollidable& him )
{
	assert( me.getCategoryBits() & (1<<COL_ENTITY_THING) );
	assert( him.getCategoryBits() & (1<<COL_ENTITY_CAR) );

	const int MAX_CONTACTS = 4;
	dContactGeom contacts[MAX_CONTACTS];
	dContact c;
	c.surface.mode = dContactSoftCFM | dContactSoftERP;
	c.surface.mu = 250;
	c.surface.soft_erp = 0.2;
	c.surface.soft_cfm = 0.0003;
	int ncontacts = commonCollide( me, him, MAX_CONTACTS, contacts, c, true, true );
}

void CCommonCollisionListener::collideCar2Car( dingus::CCollidable& me, dingus::CCollidable& him )
{
	assert( me.getCategoryBits() & (1<<COL_ENTITY_CAR) );
	assert( him.getCategoryBits() & (1<<COL_ENTITY_CAR) );

	const int MAX_CONTACTS = 6;
	dContactGeom contacts[MAX_CONTACTS];
	dContact c;
	c.surface.mode = dContactSoftCFM | dContactSoftERP;
	c.surface.mu = 250;
	c.surface.soft_erp = 0.2;
	c.surface.soft_cfm = 0.0003;
	int ncontacts = commonCollide( me, him, MAX_CONTACTS, contacts, c, true, true );
}


void CCommonCollisionListener::collideStatic2Thing( dingus::CCollidable& me, dingus::CCollidable& him )
{
	assert( me.getCategoryBits() & (1<<COL_ENTITY_STATIC) );
	assert( him.getCategoryBits() & (1<<COL_ENTITY_THING) );
	CBasePhysicsEntity* e = reinterpret_cast<CBasePhysicsEntity*>( him.getUserData() );
	assert( e );
	dingus::CPhysicsable& physHim = e->getPhysComp().getPhysicsable();
	if( !physHim.isEnabled() )
		return;

	const int MAX_CONTACTS = 6;
	dContactGeom contacts[MAX_CONTACTS];
	dContact c;
	c.surface.mode = dContactSoftCFM | dContactSoftERP;
	c.surface.mu = 250;
	c.surface.soft_erp = 0.2;
	c.surface.soft_cfm = 0.0003;
	int ncontacts = commonCollide( me, him, MAX_CONTACTS, contacts, c, false, true );
}

void CCommonCollisionListener::collideStatic2Car( dingus::CCollidable& me, dingus::CCollidable& him )
{
	assert( me.getCategoryBits() & (1<<COL_ENTITY_STATIC) );
	assert( him.getCategoryBits() & (1<<COL_ENTITY_CAR) );
	const int MAX_CONTACTS = 8;
	dContactGeom contacts[MAX_CONTACTS];
	dContact c;
	c.surface.mode = dContactSoftCFM | dContactSoftERP | dContactApprox1 /*| dContactBounce*/;
	c.surface.mu = 1000;
	c.surface.soft_erp = 0.8;
	c.surface.soft_cfm = 0.0003;
	c.surface.bounce = 0.1f;
	c.surface.bounce_vel = 0.00f;
	int ncontacts = commonCollide( me, him, MAX_CONTACTS, contacts, c, false, true );
}
